@@ -21,7 +21,9 @@ use Params::Validate ':all';
 use Scalar::Util qw(blessed);
 use Storable qw(dclone);
 
-our $VERSION = '1.10';
+use DateTime::Format::Natural::Utils qw(trim);
+
+our $VERSION = '1.13';
 
 validation_options(
     on_fail => sub
@@ -68,10 +70,12 @@ sub _init
     $self->{Daytime} = $opts{daytime} || {};
 
     my $mod = join '::', (__PACKAGE__, 'Lang', uc $self->{Lang});
-    eval "require $mod"; die $@ if $@;
+    eval "require $mod" or die $@;
 
     $self->{data} = $mod->__new();
     $self->{grammar_class} = $mod;
+
+    $self->{format_provided} = exists $opts{format};
 }
 
 sub _init_check
@@ -99,7 +103,11 @@ sub _init_check
         format => {
             type => SCALAR,
             optional => true,
-            regex => qr!^(?:[dmy]{1,4}[-./]){2}[dmy]{1,4}$!i,
+            regex => qr!^(?:
+                           (?: (?: [dmy]{1,4}[-./] ){2}[dmy]{1,4} )
+                             |
+                           (?: [dm]{1,2}/[dm]{1,2} )
+                         )$!ix,
         },
         prefer_future => {
             # SCALARREF due to boolean.pm's implementation
@@ -279,12 +287,7 @@ sub _params_init
         (${$params->{string}}) = @_;
     }
 
-    ${$params->{string}} = do {
-        local $_ = ${$params->{string}};
-        s/^\s+//;
-        s/\s+$//;
-        $_
-    };
+    trim($params->{string});
 }
 
 sub _parse_init
@@ -574,6 +577,8 @@ sub _advance_future
 
     my $day_of_week = sub { $_[0]->_Day_of_Week(map $_[0]->{datetime}->$_, qw(year month day)) };
 
+    my $skip_weekdays = false;
+
     if ((all { /^(?:second|minute|hour)$/ } keys %modified)
         && (exists $self->{modified}{hour} && $self->{modified}{hour} == 1)
         && (($self->{Prefer_future} && $self->{datetime} <  $now)
@@ -581,7 +586,23 @@ sub _advance_future
     ) {
         $self->{postprocess}{day} = 1;
     }
-    elsif ($token_contains->('weekdays_all')
+    elsif (sub {
+        return false unless @{$self->{tokens}} == 2;
+        my ($day, $weekday) = map $self->{data}->__RE($_), qw(day weekday);
+        if ($self->{tokens}->[0] =~ $day
+         && $self->{tokens}->[1] =~ $weekday) {
+            $skip_weekdays = true;
+            return true;
+        }
+        return false;
+    }->()
+        && (all { /^(?:day|month|year)$/ } keys %modified)
+        && (($self->{Prefer_future} && $self->{datetime}->day <  $now->day)
+         || ($self->{Demand_future} && $self->{datetime}->day <= $now->day))
+    ) {
+        $self->{postprocess}{week} = 4;
+    }
+    elsif (($token_contains->('weekdays_all') && !$skip_weekdays)
         && (exists $self->{modified}{day} && $self->{modified}{day} == 1)
         && (($self->{Prefer_future} && $day_of_week->($self) <  $now->wday)
          || ($self->{Demand_future} && $day_of_week->($self) <= $now->wday))
@@ -750,7 +771,27 @@ Defaults to 'C<en>'.
 
 =item * C<format>
 
-Specifies the format of numeric dates, defaults to 'C<d/m/y>'.
+Specifies the format of numeric dates.
+
+The format is used to influence how numeric dates are parsed. Given two
+numbers separated by a slash, the month/day order expected comes from
+this option. If there is a third number, this option describes where
+to expect the year. When this format can't be used to interpret the
+date, some unambiguous dates may be parsed, but there is no form
+guarantee.
+
+Current supported "month/day" formats: C<dd/mm>, C<mm/dd>.
+
+Current supported "year/month/day" formats (with slashes): C<dd/mm/yy>,
+C<dd/mm/yyyy>, C<mm/dd/yyyy>, C<yyyy/mm/dd>.
+
+Note that all of the above formats with three units do also parse
+with dots or dashes as format separators.
+
+Furthermore, formats can be abbreviated as long as they remain
+unambiguous.
+
+Defaults to 'C<d/m/y>'.
 
 =item * C<prefer_future>
 
@@ -901,6 +942,7 @@ valuable suggestions and patches:
  Tim Bunce
  Ricardo Signes
  Felix Ostmann
+ Jrn Clausen
 
 =head1 SEE ALSO
 
